#ifdef HPDAT
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <unistd.h>
#include <linux/mtio.h>
#include "dds2tar.h"

typedef unsigned char byte;

typedef struct {
        int     inlen;
        int     outlen;
        byte    buf[1024];
} ioctl_arg ;

ioctl_arg arg  =
	{
		26 , 128 , { 
			0x15 , 0x10 , 0x00 , 0x00 ,
			0x14 , 0x00 , 0x00 , 0x00 ,
			0x10 , 0x00 , 0x0f , 0x0e ,
			0x40 , 0x80 , 0x00 , 0x00 ,
			0x00 , 0x20 , 0x00 , 0x00 ,
			0x00 , 0x00 , 0x00 , 0x00 ,
			0x00 , 0x00 , 0x00 , 0x00 ,
			0x00 , 0x00 , 0x00 , 0x00
		}
	};

char*sense_key[]={
        "no sense","recovered error","not ready","medium error",
        "hardware error","illegal request","unit attention","data protect",
        "blank check","vendor specific","copy aborted","aborted command",
        "equal","volume overflow","miscompare","reserved"
};


static char*tab39[]={
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	"kilobytes to data compression",
	"kilobytes from data compression",
	"kibytes to tape",
	"kilobytes from tape",
	NULL,
	NULL,
	NULL };

print_error(byte*b,int len){
        int i;
        if ( len > 32 ) len = 8+b[7] ;
        printf(" err:                ");
        for ( i = 0 ; i < len ; i ++ ) printf(" %2X",b[i]);
        printf("\n");
        printf("error sense: %s\n",sense_key[b[2]&0xf]);
        printf("asc + ascq = %2x %2x\n", b[12],b[13]);
}

void set_compression_mode(int device,int mode){
	if ( mode >= 0 ) {
		arg.buf[12] = (mode << 7) | 0x40 ;
		if ( ioctl(device,1,&arg) != 0 ) {
			close(device);
			print_error(arg.buf,arg.inlen);
			exit(1);
		}
	} else if ( mode == DDSCM_QUERY ) {
		byte *b ;
		/* query compression configuration page */
		arg.buf[0] = 0x1a ;
		arg.buf[1] = 0x00 ;
		arg.buf[2] = 0x0f ;
		arg.buf[3] = 0x00 ;
		arg.buf[4] = 0x40 ;
		arg.buf[5] = 0x00 ;
		arg.inlen = 6 ;
		arg.outlen = 128 ;
		if ( ioctl(device,1,&arg) != 0 ) {
			close(device);
			print_error(arg.buf,arg.inlen);
			exit(1);
		}
		b = arg.buf + 12 ;
		fprintf(stderr,"data compression enable %d\n",b[2]>>7);
	} else if ( mode == DDSCM_LOG ) {
		int i,j ;
		byte *b ;
		/* compression transfer log page */
		arg.buf[0] = 0x4d ;
		arg.buf[1] = 0x00 ;
		arg.buf[2] = 0x79 ;
		arg.buf[3] = 0x00 ;
		arg.buf[4] = 0x00 ;
		arg.buf[5] = 0x00 ;
		arg.buf[6] = 0x00 ;
		arg.buf[7] = 0x00 ;
		arg.buf[8] = 0x80 ;
		arg.buf[9] = 0x00 ;
		arg.buf[10] = 0x00 ;
		arg.inlen = 10 ;
		arg.outlen = 128 ;
		if ( ioctl(device,1,&arg) != 0 ) {
			close(device);
			print_error(arg.buf,arg.inlen);
			exit(1);
		}

		b = arg.buf + 0 ;
                for ( i = 0 ; i < b[3] ; ) {
                        byte*p = b+i+4 ;
                        int n,k,c ;
                        c=(p[0]<<8)+p[1] ;
                        n = p[3];
                        i += 4 + n ;
                        p += 4 ;
                        j = 0 ;
                        for ( k = 0 ; k < n ; k++ ) j = (j << 8) + p[k] ;

                        if (( c < 12 ) && ( tab39[c] != NULL ))
				printf("%s: %d\n",tab39[c],j);
                }
	} 
}
#endif
