/*
    dds2tar  Digital-Data-Storage Extract Tool

             This tool makes use of the fast seek command of DAT devices.
             Files from a selected file archive can be extracted withing
             one minute.

             J"org Weule                     weule@cs.uni-duesseldorf.de
                                             Fon: +49 211 751409

----------------------------------------------------------------------------

                              LICENCE

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
    by the Free Software Foundation; either version 1, or (at your option) 
    any	later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

----------------------------------------------------------------------------

    To anyone in germany I decalre the following:

          Ich werde gegen jede Verbreitung der Software rechtlich vorgehen,
          welche die GNU General Public Lisence (GPL) nicht eingeh"alt.

          Schon mit dem Kopieren oder Benutzen von Teilen dieser Software
          wird die GPL stillschweigend akzeptiert. Lesen Sie also die 
          Lizensbedinungen sorgf"altig! Unkenntnis sch"utzt nicht vor Strafe.

*/

/* changes
    2.0.0    spilt to three programs: dds2tar, dds2index and mt-dat.
    1.1.2    added an option to change the compression mode of the device
    1.1.0    added the options -R, -b, -e to support usage of the tar
                record listing (tar -tvR)
             removed the option -d and removed the option ON_THE_FLY
                this options cased troubles
    1.0.2    added the define ON_THE_FLY
    1.0.1    added the define FIXED_SIZE
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/mtio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include "dds2tar.h"

static char*help_text = 
"                                                                           \n"
" To create an index:                                                       \n"
"     dds2index [options... ]                                               \n"
"                                                                           \n"
"       option:                                                             \n"
"            -t F            selecting index file                           \n"
"            -e F            selecting environment file                     \n"
"            -f F            selecting device                               \n"
"            -z              writing index-file at compressed format        \n"
"            -b              set block size                                 \n"
"                                                                           \n"
" To extract files:                                                         \n"
"     dds2tar [options... ] -p prefix-list                                  \n" 
"     dds2tar [options... ] -x file-list                                    \n"
"     dds2tar [options... ] -g pat-list                                     \n"
"                                                                           \n"
"       option:                                                             \n"
"            -t F            selecting index file                           \n"
"            -R F            selecting 'tar -R' index file                  \n"
"            -f F            selecting device                               \n"
"            -z              using index-file at compressed format          \n"
"            -b #            block size ( same value as tar -b # )          \n"
"            -s #            first record number of the tar-archive         \n"
"            -v              verbose                                        \n"
"            --hash-mode                                                    \n"
"                                                                           \n"
"      You can use the index file created with dds2index or the verbose     \n"
"      tar output either. If the archive is not the first file of the tape  \n"
"      or the block size is not the default size of tar, you have to use    \n"
"      the otions -s or -b to set the correct values.                       \n"
"                                                                           \n"
" To control the dds-device                                                 \n"
"     mt-dds [-f device] <action>                                           \n"
"                                                                           \n"
"        actions:                                                           \n"
"            comp-on      set compression mode on                           \n"
"            comp-off     set compression mode off                          \n"
"            comp-query   query compression mode                            \n"
"            comp-log     print compression log page                        \n"
"            tell         print current location                            \n"
"                                                                           \n"
"                                                                           \n"
" Example:                                                                  \n"
"   1)    export TAPE=/dev/nst0                                             \n"
"         # Crate the archive ...                                           \n"
"         mt rewind                                                         \n"
"         tar c /usr/bin                                                    \n"
"                                                                           \n"
"   2)    # Create the Index ...                                            \n"
"         mt rewind                                                         \n"
"         dds2index -t myindex                                              \n"
"                                                                           \n"
"   3)    # Now we can extract files ...                                    \n"
"         # dds2tar will rewind the tape as needed.                         \n"
"         # All files /usr/bin/*mail* will be extracted.                    \n"
"         cd /                                                              \n"
"         dds2tar -t myindex -g mail | tar fxv -                            \n"
"                                                                           \n"
" Example:                                                                  \n"
"   1)    export TAPE=/dev/nst0                                             \n"
"         # Using tar with 'vvR' will give an valid index file.             \n"
"         mt rewind                                                         \n"
"         tar cvvR /usr/bin | tee mytarindex                                \n"
"                                                                           \n"
"   2)    # Now we can extract all files /usr/bin/*cc* .                    \n"
"         # dds2tar will rewind the tape as needed.                         \n"
"         # Using the tar-index will save time.                             \n"
"         dds2tar -R mytarindex -g cc | tar xfv -                           \n"
"                                                                           \n"
"   Note: If the archive is not the first file of the tape, you have to     \n"
"         use the -s option                                                 \n"

;

static char*Alpha="\n\
usage: %s [options... ] -l location-list    extracting files at locations\n\
";

    static char*version="Version 2.0.0 -- last change 1994 September 3";
/*  static char*version="Version 1.1.3 -- last change 1994 August 15"; */
/*  static char*version="Version 1.1.0 -- last change 1994 May 10"; */
/*  static char*version="Version 1.0.2 -- last change 1994 May 2";    */
/*  static char*version="Version 1.0.1 -- last change 1994 April 30"; */
/*  static char*version="Version 1.0 -- last change 1994 March 6"; */

/*-------------------------------------------------------------------------*/

/* This is the source for dds2tar, dds2index and mt-dds. The program will
   use the variable pg to decide witch version is running. The default
   can be set in the Makefile and is overwritten by the value of argv[0].
   If you give --dds2tar, --dds2index or --mt-dds as an option, the
   program will run instead of these.
*/
#ifdef PROGRAM
int pg = PROGRAM ;
#else
int pg =  DDS2TAR ;
#endif

/* --- flags --- */
int mode           = 0 ; /* mode of dds2tar */
int verbose        = 0 ;
int all            = 0 ;
int hash_mode      = 0 ;
int print_location = 0 ;
int tar_index_file = 0 ; /* use of -t or -R */
int device_change  = 0 ; /* change the mopde of the device */
int mt_action      = DDSCM_NULL ; /* turn compression on/off */

int device ;
char*device_name=NULL;

/* file io */
FILE*index_fp      = 0 ;
char*index_file    = NULL ;

/* archive location */
int tar_fb  = 0 ;		/* first block of the archive */
int tar_bs  = 20 ;		/* block size in records of 512 bytes */
int tar_n   = 10240 ;		/* block length in bytes */

/* lists of pattern to use */
char**cmp_list ;
char**location_list;

/*-------------------------------------------------------------------------*/

/* scanning location list */
static int dds_location(){
	int eb,er,sb,sr;
	while ( *location_list != NULL ) {
		sscanf(*location_list,"%u:%u-%u:%u",&sb,&sr,&eb,&er);
		extract(sb,sr,eb,er);
		location_list++ ;
	}
}

/*-------------------------------------------------------------------------*/

/* reading archive location file */

#define LOCFILETEXT "first block number is %d\n" \
"block size is %d\n" "block length is %d\n"

static int dds_read_locfile(char*n){
	FILE *fp = fopen(n,"r") ;
	if ( fp == NULL ) {
		perror("dds2tar: can not open env file ");
		exit(1);
	}
	fscanf(fp,LOCFILETEXT,&tar_fb,&tar_bs,&tar_n);
	if ( tar_n != ( 512 * tar_bs )) {
		fprintf(stderr, "wrong block length, adjusted\n");
		tar_n = tar_bs * 512 ;
	}
	fclose(fp);
	if ( verbose ) fprintf(stderr,
		" location : ( %d , %d ) \n", tar_fb , tar_bs );
}

static int dds_print_locinfo(void){
	printf(LOCFILETEXT,dds_getpos(),tar_bs,tar_n);
}

/*-------------------------------------------------------------------------*/

int main(int argc,char**argv){

	int n = 1 ;
	int compressed_mode = T_MODE ;
	int l = strlen(argv[0]);

	mode = MAKEINDEX ;


	if ( !strcmp(argv[0]+l-7,"dds2tar") )   pg = DDS2TAR ;
	if ( !strcmp(argv[0]+l-9,"dds2index") ) pg = DDS2INDEX ;
	if ( !strcmp(argv[0]+l-6,"mt-dds") )    pg = MTDDS ;

	/* scan arguments */
	while ( n < argc ) {

/*
 * Macros to test arguments ...
 */
#define T1(s)     if((!strcmp(argv[n],(s))))
#define T(s) else if((!strcmp(argv[n],(s))))
#define TT(s,t) else if((!strcmp(argv[n],(s)))||(!strcmp(argv[n],(t))))
#define MT(s,t) else\
 if( \
    ( \
     !strcmp(argv[n],(s)) \
    )||\
    ( \
     ( \
      pg == MTDDS \
     ) \
     && \
     ( \
      !strcmp(argv[n],(t)) \
     ) \
    ) \
   )
/*
 * Now scanning the arguments ...
 */

		/*
		 * misc
		 */
		T1("--help"){
			char*p=argv[0];
			fprintf(stderr,help_text,p,p,p,p,p);
			exit(0);
		}
		TT("--version","-V"){
			fprintf(stderr,"dds2tar: %s\n",version);
			exit(0);
		}
		T("-0"){
			fprintf(stderr,"dds2tar: %s\n",argv[0]);
			exit(0);
		}

		TT("--verbose","-v")   verbose = 1 ;
		T("--hash-mode")       hash_mode = 1 ;
		T("-a")                all = 1 ;

		/*
		 * Mode of the program
		 */
		T("--dds2index")       pg = DDS2INDEX ;
		T("--dds2tar")         pg = DDS2TAR ;
		T("--mt-dds")          pg = MTDDS ;

		/*
		 * device name
		 */
		T("-f")                   device_name = argv[++n] ;

		/*
		 * Location of the file
		 */
		MT("-L","tell")           pg = MTDDS , print_location = 1 ;
		T("-e")                   dds_read_locfile(argv[++n]);
		TT("-s","--first-block")  tar_fb = atoi(argv[++n]) ;
		TT("--block-size","-b"){
			tar_bs = atoi(argv[++n]) ;
			tar_n = tar_bs * 512 ;
		}

		/*
		 * index file
		 */
		T("-t"){
			index_file = argv[++n] ;
			tar_index_file = 0 ;
		}
		T("-R"){
			index_file = argv[++n] ;
			tar_index_file = 1 ;
		}
		T("-z")                   compressed_mode = C_MODE ;

		/*
		 * Tape actions ...
		 */
#ifdef HPDAT
		MT("--ddscm-on","comp-on") pg==MTDDS,mt_action= DDSCM_ON ;
		MT("--ddscm-off","comp-off") pg==MTDDS,mt_action= DDSCM_OFF ;
		MT("--ddscm-query","comp-query") pg==MTDDS,mt_action= DDSCM_QUERY ;
		MT("--ddscm-log","comp-log") pg==MTDDS,mt_action= DDSCM_LOG ;
#endif

		/*
		 * Mode of extraction ...
		 */
		T("-x"){
			cmp_list = argv+n+1 ;
			mode = EXTRACT ;
			break ;
		}
		T("-p"){
			cmp_list = argv+n+1 ;
			mode =  PREFIX ;
			break ;
		}
		T("-g"){
			cmp_list = argv+n+1 ;
			mode = GREP ;
			break ;
		}
		T("-l"){
			location_list = argv+n+1 ;
			mode = LOCATION ;
			break ;
		}

		/*
		 * Next argument ...
		 */
		n++ ;
	}

	/*
	 * Switch the program mode ... dds2tar, dds2index or mt-dds ...
	 */
	switch ( pg ) {
/*-----------------------------MT-DDS----------------------------------------*/
case MTDDS :
	device = open_device(device_name,O_WRONLY,verbose);
#ifdef HPDAT
	if ( mt_action != DDSCM_NULL ) set_compression_mode(device,mt_action);
#endif
	if ( print_location != 0 ) dds_print_locinfo();
	close(device);
break ;
/*-----------------------------DDS2TAR---------------------------------------*/
case DDS2TAR :
	device = open_device(device_name,O_RDONLY,verbose);
	index_fp = open_file(index_file,compressed_mode,"r");
	if ( verbose ) fprintf(stderr,"selecting action ... \n");
	switch(mode){
		case LOCATION: dds_location(); break;
		case EXTRACT: dds_cmp(); break;
		case PREFIX: dds_cmp(); break;
		case GREP: dds_cmp(); break;
		default: fprintf(stderr,"nothing to do ?\n");
	}
	close(device);
break;
/*-----------------------------DDS2INDEX-------------------------------------*/
case DDS2INDEX :
	device = open_device(device_name,O_RDONLY,verbose);
	index_fp = open_file(index_file,compressed_mode,"w");
	dds_index();
	close(device);
break ;
/*---------------------------------------------------------------------------*/
default:
	fprintf(stderr,"no program mode \n");
/*---------------------------------------------------------------------------*/
	}
	return 0 ;
}
