#include <stdlib.h>
#include <stdio.h>
#include <sys/mtio.h>
#include <string.h>
#include "dds2tar.h"

static tar_record b[NREC];
static int cur_byte = 0 ;

/********
	* Coverting a tar-record to an entry in the index file.
	*
	*  input: tar_record is the header of one file in the archive.
        *         The location of the header is given by the
	*         block and index value.
	* output: The returned index is the location of the next header.
	*         Normalisation of the pair (block,index) with respect to
	*         the block lenth will give the right position.
	* extern: index_fp, verbose
	*         index_fp is the FILE to write the index.
	*         verbose indicates the loggin mode.
	*
	*         If the name length of the file name is zero,
	*         and an empty block is found, the end
	*         of the archive is found and the program stops.
	*
	********************************************************************/
static int dds_index_h( tar_record * ptr , int block , int index ){
	int size;
	char*p=ptr->hdr.magic ;
	if ( tar_index_file ) {
		fprintf(stderr,"can not crate the index file,"
				" use tar -Rvt instead\n");
		exit(1);
	}
	if ( *p == 0 ) p ="magic " ;
	else if ( p[6] == ' ' ) p[6] = 0 ;
	if ( ptr->hdr.name[0] == 0 ) {
		int i = 0 ;
		while ( ( ((char*)ptr)[i] == 0 ) && ( i < 512 )) i++ ;
		if ( i == 512 ) {
			fprintf(index_fp,"%6s%7d%3d:%9d %s\n",
				"-end- ",
				block,
				index,
				0,"(null)");
			close(device);
			exit(0);
			if ( hash_mode == 1 ) {
				fprintf(stderr," %d\n",cur_byte);
				fflush(stderr);
			}
		}
	}
	if ( ptr->hdr.size[0] != 0 ) sscanf(ptr->hdr.size,"%o",&size);
	else size = 0 ;
	ptr->hdr.name[100]=0;
	fprintf(index_fp,"%6s%7d%3d:%9d %s\n",
		p,
		block,
		index,
		size,
		ptr->hdr.name
	);
	if ( verbose ) 
		fprintf(stderr,"%6s%7d%3d:%9d %s\n",
		p,
		block,
		index,
		size,
		ptr->hdr.name
		);
	/* calculate the number of records */
	size += 511 ; size >>= 9 ;
	index += size ;
	/* there was one header record */
	index ++ ;
	return index ;
}

/* procedure to create the index table */
/********
	*  input: none
	* output: none
	* extern: device, index_fp
	*
	* The archive is read, and the index file is written.
	* If dump_mode if set, the archive is read from stdin and
	* written to 'device' with a block size of 64 tar-records.
	*
	************************************************************/

int dds_index(void){

	int i;
	int n;
	int cur_blkno ;
	static const char hash_sign = '#' ;
	static const char eoline = '\n' ;

	cur_blkno = dds_getpos();
	i = 0 ;
	fprintf(index_fp,"magic record blk:     size name\n");

	n = read(device,b,N);
	cur_byte = n ;
	if ( n == 0 ) {
		/* filemark found ? */
		n = read(device,b,N);
		cur_byte += n ;
	}
	if ( hash_mode == 1 ) if ( (cur_blkno & 0x1f) == 0 ){
		write(2,&hash_sign,1);
	}

	while ( n > 0 ){
		int bs ;
		if (( n & 0x1ff ) != 0 ) {
			fprintf(stderr,
			"tape record size is not a multiple of tar records\n"
			);
			close(device);
			exit(1);
		}
		bs = n >> 9 ;
		while ( i < bs ) i = dds_index_h(b+i,cur_blkno,i);
		i -= bs ;
#ifdef HPDAT4
		/*
		 * The hp-DAT is slowed down with dds_getpos().
                 * Is there a reason for this?
		 */
		cur_blkno++ ;
#else
		/*
		 * I think this is more save :
		 */
		cur_blkno = dds_getpos();
#endif
		n = read(device,b,N);
		cur_byte += n ;
		if ( hash_mode == 1 ) if ((cur_blkno & 0x1f)==0){
			if ( (cur_blkno & 0x7ff ) == 0 ) {
				fprintf(stderr," %d\n",
					cur_byte);
				fflush(stderr);
			}
			write(2,&hash_sign,1);
		}
	}
	if ( n <= 0 ) {
		perror("dds2tar");
		fprintf(stderr,"dds_index: unexpected end of tar archive \n");
		close(device);
		exit(1);
	}
}

